# -*- coding: utf-8 -*-
#
# sathub/views.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import collections
import platform

import flask

from flask import abort
from flask import flash
from flask import redirect
from flask import request
from flask import render_template
from flask import url_for

from flask.ext.login import LoginManager
from flask.ext.login import UserMixin
from flask.ext.login import current_user
from flask.ext.login import login_user
from flask.ext.login import login_required
from flask.ext.login import logout_user

from . import __version__ as sathub_version
from . import app
from . import executor
from . import sathubconf
from .forms import EmptyForm
from .forms import LoginForm


FUNCOES_ABERTAS = {
        'consultarsat': dict(
                titulo=u'Consultar SAT',
                descricao=u'Testa a comunicação com o equipamento SAT',
                funcao='ConsultarSAT'),

        'consultarstatusoperacional': dict(
                titulo=u'Status Operacional',
                descricao=u'Consulta o status operacional do equipamento SAT',
                funcao='ConsultarStatusOperacional'),
    }


FUNCOES_RESTRITAS = {
        'extrairlogs': dict(
                titulo=u'Extrair Logs',
                descricao=u'Obtém os registros de log do equipamento SAT.',
                funcao='ExtrairLogs'),
    }


class User(UserMixin):

    @staticmethod
    def get(user_id):
        if sathubconf.usuario != user_id:
            return None
        user = User()
        user.id = sathubconf.usuario
        return user


    @staticmethod
    def authenticate(username, password):
        if sathubconf.usuario == username:
            if sathubconf.senha == password:
                user = User()
                user.id = sathubconf.usuario
                return user
        return None


login_manager = LoginManager()
login_manager.init_app(app)


@app.context_processor
def injetar_extrainfo():
    return dict(
            sathubconf=sathubconf,
            flask_version=flask.__version__,
            python_version=platform.python_version(),
            platform_uname=' | '.join(platform.uname()),
            produto=dict(nome='SATHub', versao=sathub_version))


@app.errorhandler(404)
def pagina_nao_encontrada(error):
    return render_template('404.html'), 404


@login_manager.user_loader
def load_user(user_id):
    return User.get(user_id)


@login_manager.unauthorized_handler
def unauthorized():
    return render_template('401.html'), 401


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/login', methods=['GET', 'POST'])
def login():
    form = LoginForm(request.form)
    if request.method == 'POST' and form.validate():
        user = User.authenticate(form.username.data, form.password.data)
        if user:
            login_user(user)
            flash('Login realizado com sucesso.', 'success')
            return redirect(request.args.get('next') or url_for('index'))
        else:
            form.username.errors.append('athentication error')

    return render_template('login.html', form=form)


@app.route('/logout')
@login_required
def logout():
    logout_user()
    flash(u'Você acabou de fazer o logout.', 'info')
    return redirect(url_for('index'))


@app.route('/exec/<funcaosat>', methods=['GET', 'POST'])
def executar_funcao_sat(funcaosat):

    resultado = None

    if funcaosat in FUNCOES_RESTRITAS:
        if not current_user.is_authenticated:
            return app.login_manager.unauthorized()
        funcao = FUNCOES_RESTRITAS.get(funcaosat)
    else:
        funcao = FUNCOES_ABERTAS.get(funcaosat)
        if not funcao:
            abort(404)

    form = funcao.get('form_class', EmptyForm)(request.form)
    if request.method == 'POST' and form.validate():
        resultado = getattr(executor, funcao.get('funcao').lower())(form)

    return render_template('funcao.html',
            funcao=funcao,
            form=form,
            resultado=resultado)



# -*- coding: utf-8 -*-
#
# sathub/resources/associarassinatura.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import logging

from flask.ext import restful

from ..comum.util import hexdump
from ..comum.util import instanciar_funcoes_sat
from ..custom import request_parser


logger = logging.getLogger('sathub.resource')

parser = request_parser()

parser.add_argument('sequencia_cnpj',
        type=str,
        required=True,
        help=u'Sequencia contendo CNPJ da AC e do estabelecimento comercial')

parser.add_argument('assinatura_ac',
        type=str,
        required=True,
        help=u'Assinatura da sequencia pela AC codificada em base64')


class AssociarAssinatura(restful.Resource):

    def post(self):
        args = parser.parse_args()

        numero_caixa = args['numero_caixa']
        sequencia_cnpj = args['sequencia_cnpj']
        assinatura_ac = args['assinatura_ac']

        fsat = instanciar_funcoes_sat(numero_caixa)
        retorno = fsat.associar_assinatura(sequencia_cnpj, assinatura_ac)

        if logger.isEnabledFor(logging.DEBUG):
            logger.debug('Retorno "AssociarAssinatura" ('
                    'numero_caixa=%s, '
                    'sequencia_cnpj="%s", '
                    'assinatura_ac="%s")\n%s',
                            numero_caixa,
                            sequencia_cnpj,
                            assinatura_ac,
                            hexdump(retorno))

        return dict(funcao='AssociarAssinatura', retorno=retorno)



# -*- coding: utf-8 -*-
#
# sathub/executor.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import sys
import traceback

from unidecode import unidecode

from satcfe.excecoes import ErroRespostaSATInvalida
from satcfe.excecoes import ExcecaoRespostaSAT
from satcfe.resposta import RespostaSAT

from . import app
from .comum import util


CAIXA = 999


class ResultadoFuncao(object):

    def __init__(self, funcao, resposta=None):
        self.funcao = funcao
        self.resposta = resposta
        self._exc_info = sys.exc_info()

        if not self.resposta:
            if self.exc_type == ExcecaoRespostaSAT:
                self.resposta = self.exc_value


    @property
    def sucesso(self):
        if self.exc_type:
            return False
        return True


    @property
    def exc_type(self):
        return self._exc_info[0]


    @property
    def exc_value(self):
        return self._exc_info[1]


    @property
    def exc_traceback(self):
        return self._exc_info[2]


    @property
    def traceback(self):
        if self.exc_traceback:
            return ''.join(traceback.format_exception(
                    self.exc_type, self.exc_value, self.exc_traceback))
        return ''



def _executar(funcao, metodo, *args, **kwargs):
    try:
        cliente = util.instanciar_cliente_local(CAIXA)
        resposta = getattr(cliente, metodo)(*args, **kwargs)
        return ResultadoFuncao(funcao, resposta=resposta)

    except ErroRespostaSATInvalida as einv:
        app.logger.exception('Erro executando "%s"', funcao)
        return ResultadoFuncao(funcao)

    except ExcecaoRespostaSAT as exc_resp:
        app.logger.exception('Erro executando "%s"', funcao)
        return ResultadoFuncao(funcao)

    except Exception as exc:
        app.logger.exception('Erro executando "%s"', funcao)
        return ResultadoFuncao(funcao)


def consultarsat(form):
    return _executar('ConsultarSAT', 'consultar_sat')


def consultarstatusoperacional(form):
    return _executar('ConsultarStatusOperacional',
            'consultar_status_operacional')


def extrairlogs(form):
    resultado = _executar('ExtrairLogs', 'extrair_logs')
    if resultado.sucesso: # (!) alerta para 'quick fix'
        resultado.conteudo_log = unidecode(resultado.resposta.conteudo())
    return resultado



# -*- coding: utf-8 -*-
#
# sathub/forms.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from flask_wtf import Form
from wtforms import StringField
from wtforms import PasswordField
from wtforms import validators


LOCALES = ['pt_BR', 'pt']


class LoginForm(Form):

    username = StringField(u'Nome de Usuário',
            validators=[
                    validators.required(),
                    validators.length(min=2, max=20),])

    password = PasswordField('Senha',
            validators=[
                    validators.required(),
                    validators.length(min=6, max=20),])

    class Meta:
        locales = LOCALES


class EmptyForm(Form):
    class Meta:
        locales = LOCALES



# -*- coding: utf-8 -*-
#
# sathub/__init__.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

__version__ = '0.2'

from sathub.comum.config import conf as sathubconf
from satcfe import conf as satcfeconf

satcfeconf.codigo_ativacao = sathubconf.codigo_ativacao
sathubconf.descrever()

from flask import Flask

app = Flask(__name__)
app.secret_key = 'GTkjQdFVtRbmQpc0CoUAFiBUCSpLujtd'
app.debug = sathubconf.debug

import sathub.api
import sathub.views
